package Model.Movimientos;

import Model.Enumerate.*;
import Model.Tablero;
import Model.Exceptions.ColumnaIncorrecta;


public class MovimientoComplica extends Movimiento {
	
	private Ficha fichaExpulsada;
	
	//Constructor
	public MovimientoComplica(int columna, Ficha turno){
		super(columna, turno);
		
	}
	
	//Metodos accedentes y mutadores
	public Ficha getFichaExpulsada(){
		
		return this.fichaExpulsada;
		
	}
	
	public void setFichaExpulsada(Ficha color){
		
		this.fichaExpulsada = color;
		
	}
	
	//Metodos
	private boolean columnaCorrecta(Tablero tablero){
		
		boolean columnaValida = tablero.columnaCorrecta(this.getColumna());
		return columnaValida;
	}
	
	private int devuelveFilaLibre(int columna, Tablero tablero){
		boolean filaEncontrada = false;
		int fila = 0;
		while((fila < tablero.getAlto())  && (!filaEncontrada)){
			if(tablero.getFicha(fila, columna) == Ficha.VACIA){
				filaEncontrada = true;
			}
			else{
				fila++;
			}
		}
		
		if(fila == tablero.getAlto()){
			fila = -1;
		}
		return fila;
		
	}
	
	private void bajarFichas(int columna, Tablero tablero){
		
		Ficha color = tablero.getFicha(tablero.getAlto()-1, columna);
		for(int fila = tablero.getAlto()-2; fila >= 0; fila-- ){
			Ficha colorAux = tablero.getFicha(fila, columna);
			tablero.setFicha(fila, columna, color);
			color = colorAux;
		}
		
	}

	@Override
	public void ejecutaMovimiento(Tablero tablero) throws ColumnaIncorrecta {
		// TODO Auto-generated method stub
		if(!this.columnaCorrecta(tablero)){
			throw new ColumnaIncorrecta(tablero.getAncho());
			
		}
		else{
			//Guardamos la fila en la que colocaremos la ficha
			this.setFila(this.devuelveFilaLibre(this.getColumna(), tablero));
			//Comprobamos si la fila esta en los limites del tablero o hay que bajar fichas para ponerla en la ultima fila
			if(this.getFila() != -1){
				//Coloco la ficha en la posicion
				tablero.setFicha(this.getFila(), this.getColumna(), this.getTurno());
			}
			else{
				//Guardar la ficha de la fila 0 en una variable, bajar las ficha una fila-1
				this.setFichaExpulsada(tablero.getFicha(0, this.getColumna()));
				this.bajarFichas(this.getColumna(), tablero);
				//Coloco la ficha en la ultima fila de la columna del tablero
				tablero.setFicha(tablero.getAlto()-1, this.getColumna(), this.getTurno());
				
			}
			
		}
		
	}
	
	private void subirFichas(int columna, Tablero tablero){
		
		Ficha color = tablero.getFicha(0, columna);
		for(int fila = 1; fila <= tablero.getAlto()-1; fila++){
			Ficha colorAux = tablero.getFicha(fila, columna);
			tablero.setFicha(fila, columna, color);
			color = colorAux;
		}
		
	}

	@Override
	public void undo(Tablero tablero) {
		// TODO Auto-generated method stub
		if(this.getFichaExpulsada() != Ficha.VACIA){
			//Subimos todas las fichas de la columna una fila+1
			this.subirFichas(this.getColumna(), tablero);
			//Coloco en la fila 0 la ficha que se habia expulsado
			tablero.setFicha(0, this.getColumna(), this.getFichaExpulsada());
		}
		else{
			tablero.setFicha(this.getFila(), this.getColumna(), Ficha.VACIA);
		}

	}

}
