package Model.Movimientos;

import Model.Enumerate.*;
import Model.Tablero;
import Model.Exceptions.ColumnaIncorrecta;
import Model.Exceptions.ColumnaLlena;
import Model.Exceptions.ErrorDeEjecucion;
import Model.Exceptions.FilaIncorrecta;

public class MovimientoGravity extends Movimiento {
	
	//Constructor
	public MovimientoGravity(int fila, int columna, Ficha turno){
		
		super(fila, columna, turno);
		
	}
	
	private boolean columnaCorrecta(Tablero tablero){
		
		return tablero.columnaCorrecta(this.getColumna());
		
	}
	
	private boolean filaCorrecta(Tablero tablero){
		
		return tablero.filaCorrecta(this.getFila());
		
	}
	
	private void desplazaArriba(Tablero tablero){
		
		int filaProvisional = this.getFila()+1;
		boolean posicionOcupada = false;
		while((filaProvisional <= tablero.getAlto()) && (!posicionOcupada)){
			if(tablero.getFicha(filaProvisional, this.getColumna()) == Ficha.VACIA){
				filaProvisional++;
			}
			else{
				posicionOcupada = true;
			}
		}
		this.setFila(filaProvisional-1);
		
	}
	
	private void desplazarAbajo(Tablero tablero){
		
		int filaProvisional = this.getFila()-1;
		boolean posicionOcupada = false;
		while((filaProvisional >= 1) && (!posicionOcupada)){
			if(tablero.getFicha(filaProvisional, this.getColumna()) == Ficha.VACIA){
				filaProvisional--;
			}
			else{
				posicionOcupada = true;
			}
		}
		this.setFila(filaProvisional+1);
		
	}
	
	private void desplazarDerecha(Tablero tablero){
		
		int columnaProvisional = this.getColumna()+1;
		boolean posicionOcupada = false;
		while((columnaProvisional <= tablero.getAncho()) && (!posicionOcupada)){
			if(tablero.getFicha(this.getFila(), columnaProvisional) == Ficha.VACIA){
				columnaProvisional++;
			}
			else{
				posicionOcupada = true;
			}
		}
		this.setColumna(columnaProvisional-1);
		
	}
	
	private void desplazarIzquierda(Tablero tablero){
		
		int columnaProvisional = this.getColumna()-1;
		boolean posicionOcupada = false;
		while((columnaProvisional >= 1) && (!posicionOcupada)){
			if(tablero.getFicha(this.getFila(), columnaProvisional) == Ficha.VACIA){
				columnaProvisional--;
			}
			else{
				posicionOcupada = true;
			}
		}
		this.setColumna(columnaProvisional+1);
		
	}
	
	private void desplazarArribaDerecha(Tablero tablero){
		
		int filaProvisional = this.getFila()+1;
		int columnaProvisional = this.getColumna()+1;
		boolean posicionOcupada = false;
		while((filaProvisional <= tablero.getAlto()) && (columnaProvisional <= tablero.getAncho()) && (!posicionOcupada)){
			if(tablero.getFicha(filaProvisional, columnaProvisional) == Ficha.VACIA){
				filaProvisional++;
				columnaProvisional++;
			}
			else{
				posicionOcupada = true;
			}
		}
		this.setFila(filaProvisional-1);
		this.setColumna(columnaProvisional-1);
		
	}
	
	private void desplazarArribaIzquierda(Tablero tablero){
		
		int filaProvisional = this.getFila()+1;
		int columnaProvisional = this.getColumna()-1;
		boolean posicionOcupada = false;
		while((filaProvisional <= tablero.getAlto()) && (columnaProvisional >= 1) && (!posicionOcupada)){
			if(tablero.getFicha(filaProvisional, columnaProvisional) == Ficha.VACIA){
				filaProvisional++;
				columnaProvisional--;
			}
			else{
				posicionOcupada = true;
			}
		}
		this.setFila(filaProvisional-1);
		this.setColumna(columnaProvisional+1);
		
	}
	
	private void desplazarAbajoDerecha(Tablero tablero){
		
		int filaProvisional = this.getFila()-1;
		int columnaProvisional = this.getColumna()+1;
		boolean posicionOcupada = false;
		while((filaProvisional >= 1) && (columnaProvisional <= tablero.getAncho()) && (!posicionOcupada)){
			if(tablero.getFicha(filaProvisional, columnaProvisional) == Ficha.VACIA){
				filaProvisional--;
				columnaProvisional++;
			}
			else{
				posicionOcupada = true;
			}
		}
		this.setFila(filaProvisional+1);
		this.setColumna(columnaProvisional-1);
		
	}
	
	private void desplazarAbajoIzquierda(Tablero tablero){
		
		int filaProvisional = this.getFila()-1;
		int columnaProvisional = this.getColumna()-1;
		boolean posicionOcupada = false;
		while((filaProvisional >= 1) && (columnaProvisional >= 1) && (!posicionOcupada)){
			if(tablero.getFicha(filaProvisional, columnaProvisional) == Ficha.VACIA){
				filaProvisional--;
				columnaProvisional--;
			}
			else{
				posicionOcupada = true;
			}
		}
		this.setFila(filaProvisional+1);
		this.setColumna(columnaProvisional+1);
		
	}
	
	private void desplazaFicha(Tablero tablero){
		
		int distanciaMinima;
		int distanciaArriba = tablero.getAlto()-this.getFila();
		int distanciaAbajo = this.getFila()-1;
		int distanciaDerecha = tablero.getAncho()-this.getColumna();
		int distanciaIzquierda = this.getColumna()-1;
		//Calculamos la distancia minima a cualquiera de las paredes del tablero.
		distanciaMinima = Math.min(distanciaArriba, distanciaAbajo);
		distanciaMinima = Math.min(distanciaMinima, distanciaDerecha);
		distanciaMinima = Math.min(distanciaMinima, distanciaIzquierda);
		//Realizamos el movimiento de la ficha dependiendo segun la distancia minima.
		if((distanciaMinima == distanciaArriba) && (distanciaMinima != distanciaAbajo) && 
				(distanciaMinima != distanciaDerecha) && (distanciaMinima != distanciaIzquierda)){
			this.desplazaArriba(tablero);
		}
		else if((distanciaMinima != distanciaArriba) && (distanciaMinima == distanciaAbajo) && 
				(distanciaMinima != distanciaDerecha) && (distanciaMinima != distanciaIzquierda)){
			this.desplazarAbajo(tablero);
		}
		else if((distanciaMinima != distanciaArriba) && (distanciaMinima != distanciaAbajo) && 
				(distanciaMinima == distanciaDerecha) && (distanciaMinima != distanciaIzquierda)){
			this.desplazarDerecha(tablero);
		}
		else if((distanciaMinima != distanciaArriba) && (distanciaMinima != distanciaAbajo) && 
				(distanciaMinima != distanciaDerecha) && (distanciaMinima == distanciaIzquierda)){
			this.desplazarIzquierda(tablero);
		}
		else if((distanciaMinima == distanciaArriba) && (distanciaMinima != distanciaAbajo) && 
				(distanciaMinima == distanciaDerecha) && (distanciaMinima != distanciaIzquierda)){
			this.desplazarArribaDerecha(tablero);
		}
		else if((distanciaMinima != distanciaArriba) && (distanciaMinima == distanciaAbajo) && 
				(distanciaMinima == distanciaDerecha) && (distanciaMinima != distanciaIzquierda)){
			this.desplazarAbajoDerecha(tablero);
		}
		else if((distanciaMinima == distanciaArriba) && (distanciaMinima != distanciaAbajo) && 
				(distanciaMinima != distanciaDerecha) && (distanciaMinima == distanciaIzquierda)){
			this.desplazarArribaIzquierda(tablero);
		}
		else if((distanciaMinima != distanciaArriba) && (distanciaMinima == distanciaAbajo) && 
				(distanciaMinima != distanciaDerecha) && (distanciaMinima == distanciaIzquierda)){
			this.desplazarAbajoIzquierda(tablero);
		}
		else if((distanciaMinima == distanciaArriba) && (distanciaMinima == distanciaAbajo) && 
				(distanciaMinima == distanciaDerecha) && (distanciaMinima == distanciaIzquierda)){
			this.desplazarAbajo(tablero);
		}
		
	}

	@Override
	public void ejecutaMovimiento(Tablero tablero) throws ColumnaLlena, FilaIncorrecta, ColumnaIncorrecta, ErrorDeEjecucion {
		// TODO Auto-generated method stub
		if(!this.columnaCorrecta(tablero)){
			throw new ColumnaIncorrecta(tablero.getAncho());
			
		}
		else if(!this.filaCorrecta(tablero)){
			throw new FilaIncorrecta(tablero.getAlto());
			
		}
		else if(tablero.getFicha(this.getFila(), this.getColumna()) != Ficha.VACIA){
			throw new ErrorDeEjecucion("Esta posición ya está ocupada.");
			
		}
		else{
			//Realizamos el movimiento
			this.desplazaFicha(tablero);
			//Ponemos la ficha en el lugar correcto del tablero.
			tablero.setFicha(this.getFila(), this.getColumna(), this.getTurno());
			
		}
		
	}

	@Override
	public void undo(Tablero tablero) {
		// TODO Auto-generated method stub
		tablero.setFicha(this.getFila(), this.getColumna(), Ficha.VACIA);

	}

}
