package Model.Reglas;

import Model.Enumerate.*;
import Model.Tablero;

public final class ReglasJuegoCuatroEnLinea {
	
	private static final int LINEA = 4;
	
	static private boolean cuatroEnVertical(Tablero tablero, Ficha turno, int fila, int columna){
		
		boolean hayCuatroEnVertical = false;
		int fichasIguales = 0;
		int filaInicial = fila;
		int limiteDeBusquedas = 0;
		while((fila < tablero.getAlto()) && (limiteDeBusquedas < LINEA)){
			if(tablero.getFicha(fila, columna) == turno){
				fichasIguales++;
				fila++;
				limiteDeBusquedas++;
			}
			else{
				limiteDeBusquedas = LINEA;
			}
		}
		
		fila = filaInicial-1;
		limiteDeBusquedas = 0;
		while((fila >= 0) && (limiteDeBusquedas < LINEA)){
			if(tablero.getFicha(fila, columna) == turno){
				fichasIguales++;
				fila--;
				limiteDeBusquedas++;
			}
			else{
				limiteDeBusquedas = LINEA;
			}
		}
		//Comprobamos si hay cuatro en linea
		if(fichasIguales == LINEA){
			hayCuatroEnVertical = true;
		}
		
		return hayCuatroEnVertical;
	}
	
	static private boolean cuatroEnHorizontal(Tablero tablero, Ficha turno, int fila, int columna){
		
		boolean hayCuatroEnHorizontal = false;
		int fichasIguales = 0;
		int columnaInicial = columna;
		
		int limiteDeBusquedas = 0;
		while((columna < tablero.getAncho()) && (limiteDeBusquedas < LINEA)){
			if(tablero.getFicha(fila, columna) == turno){
				fichasIguales++;
				columna++;
				limiteDeBusquedas++;
			}
			else{
				limiteDeBusquedas = LINEA;
			}
		}
		
		columna = columnaInicial-1;
		limiteDeBusquedas = 0;
		while((columna >= 0) && (limiteDeBusquedas < LINEA)){
			if(tablero.getFicha(fila, columna) == turno){
				fichasIguales++;
				columna--;
				limiteDeBusquedas++;
			}
			else{
				limiteDeBusquedas = LINEA;
			}
		}
		//Comprobamos si hay cuatro en linea
		if(fichasIguales == LINEA){
			hayCuatroEnHorizontal = true;
		}
		
		return hayCuatroEnHorizontal;
		
	}
	
	static private boolean cuatroEnDiagonalADerecha(Tablero tablero, Ficha turno, int fila, int columna){
		
		boolean hayCuatroEnDiagonalADerecha = false;
		int fichasIguales = 0;
		int filaInicial = fila;
		int columnaInicial = columna;
		
		int limiteDeBusquedas = 0;
		while((fila < tablero.getAlto()) && (columna < tablero.getAncho()) && (limiteDeBusquedas < LINEA)){
			if(tablero.getFicha(fila, columna) == turno){
				fichasIguales++;
				limiteDeBusquedas++;
				fila++;
				columna++;
			}
			else{
				limiteDeBusquedas = LINEA;
			}
		}
		
		fila = filaInicial-1;
		columna = columnaInicial-1;
		limiteDeBusquedas = 0;
		while((fila >= 0) && (columna >= 0) && (limiteDeBusquedas < LINEA)){
			if(tablero.getFicha(fila, columna) == turno){
				fichasIguales++;
				limiteDeBusquedas++;
				fila--;
				columna--;
			}
			else{
				limiteDeBusquedas = LINEA;
			}
		}
		//Comprobamos si hay cuatro en linea
		if(fichasIguales == LINEA){
			hayCuatroEnDiagonalADerecha = true;
		}
		
		return hayCuatroEnDiagonalADerecha;
		
	}
	
	static private boolean cuatroEnDiagonalAIzquierda(Tablero tablero, Ficha turno, int fila, int columna){
		
		boolean hayCuatroEnDiagonalAIzquierda = false;
		int fichasIguales = 0;
		int filaInicial = fila;
		int columnaInicial = columna;
		
		int limiteDeBusquedas = 0;
		while(fila < tablero.getAlto() && (columna >= 0) && (limiteDeBusquedas < LINEA)){
			if(tablero.getFicha(fila, columna) == turno){
				fichasIguales++;
				limiteDeBusquedas++;
				fila++;
				columna--;
			}
			else{
				limiteDeBusquedas = LINEA;
			}
		}
		
		fila = filaInicial-1;
		columna = columnaInicial+1;
		limiteDeBusquedas = 0;
		while((fila >= 0)  && (columna < tablero.getAncho()) && (limiteDeBusquedas < LINEA)){
			if(tablero.getFicha(fila, columna) == turno){
				fichasIguales++;
				limiteDeBusquedas++;
				fila--;
				columna++;
			}
			else{
				limiteDeBusquedas = LINEA;
			}
		}
		//Comprobamos si hay cuatro en linea
		if(fichasIguales == LINEA){
			hayCuatroEnDiagonalAIzquierda = true;
		}
		
		return hayCuatroEnDiagonalAIzquierda;
		
	}
	
	static public boolean cuatroEnLinea(Tablero tablero, Ficha turno, int fila, int columna){
		
		boolean cuatroEnLinea = false;
		if((cuatroEnVertical(tablero, turno, fila, columna)) || 
				(cuatroEnHorizontal(tablero, turno, fila, columna)) ||
				(cuatroEnDiagonalADerecha(tablero, turno, fila, columna)) || 
				(cuatroEnDiagonalAIzquierda(tablero, turno, fila, columna))){
			
			cuatroEnLinea = true;
		}
		return cuatroEnLinea;
	}

}
